#include "displaynmwidget.h"
#include <NetworkManagerQt/Manager>
#include <QFrame>
#include <KWindowEffects>
#include <QBitmap>
#include <QPainter>
#include <QProcess>
#include "icons.h"
#include <QApplication>
#include <QScreen>
#include <QDBusInterface>

DisplayNMWidget::DisplayNMWidget(QWidget *parent) : QWidget(parent)
{
    //适配最小分辨率
    QRect availableRect = qApp->primaryScreen()->availableGeometry();
    QSize fixedSize = QSize(qMin(availableRect.width(), 312), qMin(availableRect.height(), 412) - 4);//高度留4px的间隔
    this->setFixedSize(fixedSize);
    this->setObjectName("DisplayNMWidget");
    this->setWindowFlags(Qt::FramelessWindowHint |
                         Qt::WindowStaysOnTopHint |
                         Qt::Tool |
                         Qt::X11BypassWindowManagerHint);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);

    KWindowEffects::enableBlurBehind(this->winId(), true);

    m_labelTitle = new QLabel(this);
    m_labelTitle->setObjectName("label_title");
    m_labelTitle->setText(tr("NetWork"));
    m_labelTitle->setFixedHeight(32);
    QFont f = m_labelTitle->font();
    f.setPixelSize(16);
    m_labelTitle->setFont(f);

    m_btnSetting = new QToolButton(this);
    m_btnSetting->setEnabled(true);
    m_btnSetting->setFixedSize(30,30);
    m_btnSetting->setObjectName("btnSetting");
    m_btnSetting->setIconSize(QSize(30,30));
    connect(m_btnSetting, &QToolButton::clicked, this,&DisplayNMWidget::onBtnSettingClicked);

    QHBoxLayout * titleLayout = new QHBoxLayout();
    titleLayout->setSpacing(10);
    titleLayout->setMargin(0);
    titleLayout->setContentsMargins(12,10,12,10);
    titleLayout->addWidget(m_labelTitle);
    titleLayout->addStretch();
    titleLayout->addWidget(m_btnSetting);

    int height = this->height()
            - titleLayout->contentsMargins().top() - titleLayout->contentsMargins().bottom() - m_labelTitle->height();
    m_disEmpty = new QWidget(this);
    m_disEmpty->setMinimumSize(this->width(),height);
    m_disEmpty->hide();

    m_disContain = new DisplayNMContain(this);
    m_disContain->setMinimumSize(this->width(),height);

    QFrame *hline = new QFrame(this);
    hline->setObjectName(QString::fromUtf8("hline"));
    hline->setStyleSheet(QString::fromUtf8(""));
    hline->setFrameShadow(QFrame::Sunken);
    hline->setLineWidth(1);
    hline->setMidLineWidth(0);
    hline->setFrameShape(QFrame::HLine);

    QVBoxLayout * mainLayout = new QVBoxLayout;
    mainLayout->setMargin(0);
    mainLayout->setContentsMargins(0,0,0,0);
    mainLayout->setSpacing(0);
    mainLayout->addLayout(titleLayout);
    mainLayout->addWidget(hline);
    mainLayout->addWidget(m_disEmpty);
    mainLayout->addWidget(m_disContain);
    this->setLayout(mainLayout);

    //
    m_ndeSetting = new Nde::Settings(QLatin1String("panel"), this);
    connect(m_ndeSetting, &Nde::Settings::settingsChanged, this, &DisplayNMWidget::onNdeSettingsChange);

    QDBusConnection::sessionBus().connect(QString::fromLatin1("org.nde.session"),
                                          QString::fromLatin1("/NdeSession"),
                                          QString::fromLatin1("org.nde.session"),
                                          QString::fromLatin1("compositorOpenStateChanged"),
                                          this, SLOT(onCompositorOpenChanged(bool)));

    QDBusInterface * compositer = new QDBusInterface(QString::fromLatin1("org.nde.session"),
                                                     QString::fromLatin1("/NdeSession"),
                                                     QString::fromLatin1("org.nde.session"),
                                                     QDBusConnection::sessionBus());
    if(compositer->isValid())
    {
        QDBusMessage reply = compositer->call("getCompositorOpen");
        if(!reply.arguments().isEmpty()) mCompositorOpen = reply.arguments().first().toBool();
    }

    //
    for (auto const & device : NetworkManager::networkInterfaces())
    {
        //该对象只获取有线和无线设备
        qWarning() << "DEVICE INFORMATION:" << device->interfaceName() << " "<< device->driver() <<" "<< device->driverVersion() << device->type();
        if(device->type() == NetworkManager::Device::Ethernet || device->type() == NetworkManager::Device::Wifi)
        {
            m_disContain->addDevice(device);
            mDevices.append(device);
        }
    }
    onNdeSettingsChange();

    mScreenRect = qApp->primaryScreen()->geometry();
    connect(qApp->primaryScreen(),  &QScreen::geometryChanged, this, &DisplayNMWidget::onScreenSizeChanged);
}

void DisplayNMWidget::showDisplayNMWidget()
{
    if(this->isVisible())
    {
        this->hide();
    }
    else {

        this->show();
        this->activateWindow();

        QPoint p = QCursor::pos();
        QPoint newP = p;
        QRect rect = qApp->primaryScreen()->geometry();

        //    case NdePanel::PositionTop:
        //        return QString("Top");
        //    case NdePanel::PositionLeft:
        //        return QString("Left");
        //    case NdePanel::PositionRight:
        //        return QString("Right");
        //    case NdePanel::PositionBottom:
        //        return QString("Bottom");
        //    }

        if(mPanelPos == QString("Top"))
        {
            rect.adjust(0,mPanelSize,0,0);
        }
        else if(mPanelPos == QString("Left"))
        {
            rect.adjust(mPanelSize,0,0,0);
        }
        else if(mPanelPos == QString("Right"))
        {
            rect.adjust(0,0,-mPanelSize,0);
        }
        else if(mPanelPos == QString("Bottom"))
        {
            rect.adjust(0,0,0,-mPanelSize);
        }

        //mDisplayNM右侧超出
        if(rect.width() < (p.x() + this->width() + 4))
        {
            newP.rx() = rect.width()-this->width() - 4;
        }
        //mDisplayNM左侧超出
        if(p.x() - this->width() - 4 < 0)
        {
            newP.rx() = rect.left() + 4;
        }
        //mDisplayNM下面超出
        if(rect.height() < (p.y() + this->height() + 4))
        {
            newP.ry() = rect.height() - this->height() - 4;
        }
        //mDisplayNM上方超出
        if(p.y()-this->height() - 4 < 0)
        {
            newP.ry() = rect.top() + 4;
        }
        this->move(newP);
    }
}

void DisplayNMWidget::onDeviceAdded(const QString &uni)
{
    auto device = NetworkManager::findNetworkInterface(uni);
    if(device->type() == NetworkManager::Device::Ethernet || device->type() == NetworkManager::Device::Wifi)
    {
        m_disContain->addDevice(device);
        mDevices.append(device);
    }
}

void DisplayNMWidget::onDeviceRemoved(const QString &uni)
{
    auto device = NetworkManager::findNetworkInterface(uni);
    if(device->type() == NetworkManager::Device::Ethernet || device->type() == NetworkManager::Device::Wifi)
    {
        m_disContain->removeDevice(device);
        //待确认
        if(mDevices.contains(device))
        {
            mDevices.removeAll(device);
        }
    }
}

void DisplayNMWidget::onBtnSettingClicked()
{
    QProcess::startDetached("nm-connection-editor");
}

void DisplayNMWidget::onNdeSettingsChange()
{
    //mSettings
    m_ndeSetting->beginGroup("panel1");
    QString panelPos = m_ndeSetting->value("position", "Bottom").toString();
    int panelSize = m_ndeSetting->value("panelSize", 48).toInt();
    int opacity = m_ndeSetting->value("opacity",100).toInt();
    QColor bColor = m_ndeSetting->value("background-color", "").value<QColor>();
    m_ndeSetting->endGroup();

    if(panelPos != mPanelPos || panelSize != mPanelSize)
    {
        mPanelPos = panelPos;
        mPanelSize = panelSize;
    }

    mOpacity = opacity;
    mBackgroundColor = bColor;
    updateStyleSheet();
}

void DisplayNMWidget::onScreenSizeChanged()
{
    QRect rc = qApp->primaryScreen()->geometry();
    if(mScreenRect != rc)
    {
        mScreenRect = rc;
        this->hide();
    }
}

void DisplayNMWidget::onCompositorOpenChanged(bool open)
{
    mCompositorOpen = open;
    updateStyleSheet();
}

void DisplayNMWidget::updateStyleSheet()
{
    int opacity = mCompositorOpen ? mOpacity : 100;
    QColor bColor = mCompositorOpen ? mBackgroundColor : QColor("#38414a");

    QString color = QString("%1, %2, %3, %4")
            .arg(bColor.red())
            .arg(bColor.green())
            .arg(bColor.blue())
            .arg(opacity / 100.0);
    this->setStyleSheet(QString("#DisplayNMWidget {"
                                "background-color: rgba(" + color + "); }"));
}

bool DisplayNMWidget::event(QEvent *event)
{
    if(event->type() == QEvent::ActivationChange)
    {
        if(!this->isActiveWindow())
        {
            this->hide();
        }
    }
    return QWidget::event(event);
}

void DisplayNMWidget::paintEvent(QPaintEvent *event)
{
    QBitmap bmp(this->size());
    bmp.fill();
    QPainter p(&bmp);
    p.setRenderHint(QPainter::Antialiasing);
    p.setBrush(QBrush(mBackgroundColor));
    p.setPen(Qt::transparent);
    p.drawRoundedRect(this->rect(), 6, 6, Qt::AbsoluteSize); //四个角都是圆弧

    this->setMask(bmp);
}
